@@ -40,18 +40,18 @@ G_BEGIN_DECLS
 #define GTK_IS_MOUNT_OPERATION_CLASS(k)  (G_TYPE_CHECK_CLASS_TYPE ((k), GTK_TYPE_MOUNT_OPERATION))
 #define GTK_MOUNT_OPERATION_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), GTK_TYPE_MOUNT_OPERATION, GtkMountOperationClass))
 
-typedef struct GtkMountOperation         GtkMountOperation;
-typedef struct GtkMountOperationClass    GtkMountOperationClass;
-typedef struct GtkMountOperationPrivate  GtkMountOperationPrivate;
+typedef struct _GtkMountOperation         GtkMountOperation;
+typedef struct _GtkMountOperationClass    GtkMountOperationClass;
+typedef struct _GtkMountOperationPrivate  GtkMountOperationPrivate;
 
-struct GtkMountOperation
+struct _GtkMountOperation
 {
   GMountOperation parent_instance;
 
   GtkMountOperationPrivate *priv;
 };
 
-struct GtkMountOperationClass
+struct _GtkMountOperationClass
 {
   GMountOperationClass parent_class;
 
